#!/usr/bin/env ruby

puts 'Basic ISBM Pub/Sub Example with Ruby ISBM Adaptor High-Level interface'

# The ISBM v2 REST Adaptor is not yet published to rubygems.org but can be installed from
# github at: https://github.com/assetricity/isbm2_adaptor_rest.git
#
# To install the gem directly execute the following:
# > gem install specific_install
# > gem specific_install 'https://github.com/assetricity/isbm2_adaptor_rest.git'
#
# Alternatively configure a `Gemfile` as follows:
# ```
# gem 'isbm2_adaptor_rest', '~> 2.0.0', git: 'https://github.com/assetricity/isbm2_adaptor_rest.git'
# ```
# and execute:
# > bundle install
#
# If using the bundler method, don't forget to execute the code using `bundle exec ruby ...`

require 'isbm2_adaptor_rest'
require 'isbm_adaptor_common' # for common elements of high-level API such as faults/exceptions
require 'nokogiri'

IsbmRestAdaptor.configure do |conf|
  conf.host = 'isbm.au-wg.oiiecosystem.net'
  conf.scheme = 'https'
  conf.base_path = '/rest'
end

channel_management = IsbmRestAdaptor::ChannelManagement.new
channel_management.get_channels.each do | channel |
  puts channel.inspect
end

# Local config
local_config = IsbmRestAdaptor::Configuration.new.configure do |conf|
  conf.host = 'example.com'
  conf.scheme = 'http'
  conf.base_path = '/rest'
  conf.debugging = true
  conf
end

# IsbmRestAdaptor::ChannelManagement.new(client_config: local_config).get_channels

# Open Publication Session
channel_uri = '/example/hello_world/publication'
publish_client = IsbmRestAdaptor::ProviderPublication.new
publish_session_id = publish_client.open_session(channel_uri)
puts "Opened publication session: #{publish_session_id}"

subscribe_client = IsbmRestAdaptor::ConsumerPublication.new
subscribe_session_id = subscribe_client.open_session(channel_uri, 'echo-example')
puts "Opened subscription session: #{subscribe_session_id}"

begin
  expiry = IsbmAdaptor::Duration.new(hours: 24)
  # Publish message
  posted_message_id = publish_client.post_publication(publish_session_id, 'Hello world!', ['echo-example', 'example'], expiry)
  puts "Posted message: #{posted_message_id}"

  publish_client.expire_publication(publish_session_id, posted_message_id)

  publish_client.post_publication(publish_session_id, { say: 'Hello World!' }, 'echo-example')

  message = subscribe_client.read_publication(subscribe_session_id)
  puts "Read message #{message.id} from topics #{message.topics} of type #{message.media_type}:\n#{message.content}"

  subscribe_client.remove_publication(subscribe_session_id)
  puts subscribe_client.read_publication(subscribe_session_id).nil? ? 'Message removed' : 'what went wrong?'

  publish_client.post_publication(publish_session_id, Nokogiri::XML.parse('<say>Hello World!</say>'), 'echo-example')

  publish_client.post_publication(publish_session_id, { content: 'Hello World', media_type: 'text/plain' }, 'echo-example')

  while message = subscribe_client.read_publication(subscribe_session_id) do
    puts "Read message #{message.id} from topics #{message.topics} of type #{message.media_type}:\n#{message.content}"
    subscribe_client.remove_publication(subscribe_session_id)
  end

  # error handling
  begin
    posted_message_id = publish_client.post_publication('publish_session_id', 'Hello world!', 'echo-example')
  rescue IsbmAdaptor::SessionFault => ex
    puts "Posting message failed: #{ex} (cause #{ex.cause.code})"
  rescue IsbmAdaptor::ParameterFault => ex
    puts "Posting message failed, server detected invalid input: #{ex} (cause #{ex.cause.code})"
  rescue ArgumentError => ex
    # If client-side validation is enabled, this should catch most input errors from hitting the server
    puts "Client-side validation failed for attempted post publication: #{ex}"
  rescue IsbmAdaptor::UnknownFault => ex
    puts "Something bad and unexpected happended: #{ex}"
  rescue IsbmAdaptor::IsbmFault => ex
    puts "Generic catch all for ISBM specific return faults/errors: #{ex}"
  end
ensure
  publish_client.close_session(publish_session_id)
  subscribe_client.close_session(subscribe_session_id)
  puts 'Done'
end