#!/usr/bin/env ruby

# The ISBM v2 REST Adaptor is not yet published to rubygems.org but can be installed from
# github at: https://github.com/assetricity/isbm2_adaptor_rest.git
#
# To install the gem directly execute the following:
# > gem install specific_install
# > gem specific_install 'https://github.com/assetricity/isbm2_adaptor_rest.git'
#
# Alternatively configure a `Gemfile` as follows:
# ```
# gem 'isbm2_adaptor_rest', '~> 2.0.0', git: 'https://github.com/assetricity/isbm2_adaptor_rest.git'
# ```
# and execute:
# > bundle install
#
# If using the bundler method, don't forget to execute the code using `bundle exec ruby ...`

require 'isbm2_adaptor_rest'
require 'isbm_adaptor_common' # for common elements of high-level API such as faults/exceptions
require 'nokogiri'

# Configure the adaptor globally
IsbmRestAdaptor.configure do |conf|
  conf.host = 'isbm.au-wg.oiiecosystem.net'
  conf.scheme = 'https'
  conf.base_path = '/rest'
end

# Check that it works with getChannels
channel_management = IsbmRestAdaptor::ChannelManagement.new
channel_management.get_channels.each do |channel|
  puts channel.inspect
end

# Local configuration example
local_config = IsbmRestAdaptor::Configuration.new.configure do |conf|
  conf.host = 'example.com'
  conf.scheme = 'http'
  conf.base_path = '/rest'
  conf.debugging = true
  conf
end

begin
  IsbmRestAdaptor::ChannelManagement.new(client_config: local_config).get_channels
rescue IsbmAdaptor::UnknownFault => ex
  puts "#{ex}: #{local_config.host} unexpectedly returned #{ex.cause.code}"
  # puts ex.cause.response_headers
  # puts ex.cause.response_body
end
# can access the low-level API through the classes post-fixed with 'Api', 
# e.g., IsbmRestAdaptor::ChannelManagementApi
# However, we will use the high-level API in this example.

# Common values
channel_uri = '/example/hello_world/publication'
topic = 'echo-example'
text_message_content = 'Hello World!'
json_message_content = { say: 'Hello World!' }
xml_message_content = Nokogiri::XML.parse('<say>Hello World!</say>')
expiry = IsbmAdaptor::Duration.new(hours: 24)

# Open Publish Session
publish_client = IsbmRestAdaptor::ProviderPublication.new

publish_session_id = publish_client.open_session(channel_uri)
puts "Opened publication session: #{publish_session_id}"

# Open Subscribe Session
subscribe_client = IsbmRestAdaptor::ConsumerPublication.new

subscribe_session_id = subscribe_client.open_session(channel_uri, topic) # can also be list of topics
puts "Opened subscription session: #{subscribe_session_id}"

begin # ensure close

# Publish Message (String)
## Note that this part of the high-level interface will likely be updated in the near future to 
## return the entire message object, not just the ID.
posted_message_id = publish_client.post_publication(publish_session_id, text_message_content, topic)
puts "Posted message: #{posted_message_id}"

gets # wait for user to hit enter before continuing (used initially in demo when checking success via web app)

# Publish Message with error handling
begin
  posted_message_id = publish_client.post_publication('publish_session_id', text_message_content, topic)
rescue IsbmAdaptor::SessionFault => ex
  puts "Posting message failed: #{ex} (cause #{ex.cause.code})"
rescue IsbmAdaptor::ParameterFault => ex
  puts "Posting message failed, server detected invalid input: #{ex} (cause #{ex.cause.code})"
rescue ArgumentError => ex
  # If client-side validation is enabled, this should catch most input errors from hitting the server
  puts "Client-side validation failed for attempted post publication: #{ex}"
rescue IsbmAdaptor::UnknownFault => ex
  puts "Something bad and unexpected happended: #{ex}"
rescue IsbmAdaptor::IsbmFault => ex
  puts "Generic catch all for ISBM specific return faults/errors: #{ex}"
end

# Read Message
message = subscribe_client.read_publication(subscribe_session_id)
puts "Read message #{message.id} from topics #{message.topics} of type #{message.media_type}:\n#{message.content}"

# Remove Message
subscribe_client.remove_publication(subscribe_session_id)
puts subscribe_client.read_publication(subscribe_session_id).nil? ? '  Message removed' : '  Oops, something went wrong.'

# Publish JSON Message
posted_message_id = publish_client.post_publication(publish_session_id, json_message_content, [topic])

# Publish XML Message
# (with 24 hour expiry, optional parameter)
publish_client.post_publication(publish_session_id, xml_message_content, [topic], expiry)

# Expire the JSON message
publish_client.expire_publication(publish_session_id, posted_message_id)

# Republish the JSON message
publish_client.post_publication(publish_session_id, {content: json_message_content, media_type: 'application/json'}, [topic])

# Read all messages (should receive XML, then JSON)
while message = subscribe_client.read_publication(subscribe_session_id) do
  puts "Read message #{message.id} from topics #{message.topics} of type #{message.media_type}:\n#{message.content}"
  subscribe_client.remove_publication(subscribe_session_id)
end

# Close Sessions
ensure
  publish_client.close_session(publish_session_id)
  subscribe_client.close_session(subscribe_session_id)
end
